/*
 * Software Suspend header file.
 *
 * This file sets the major and minor version numbers of the version specific
 * part of the Software Suspend patch. The major and minor numbers must match
 * those of the core patch. The extra is updated as necessary for version
 * specific changes.
 */

#ifndef SWSUSP_VERSION_SPECIFIC_REVISION
#define SWSUSP_VERSION_SPECIFIC_REVISION 0x201
#define SWSUSP_VERSION_SPECIFIC_REVISION_STRING "2.0.1"

#ifdef CONFIG_BOOTSPLASH
#include <linux/console.h>
#include "../../drivers/video/console/fbcon.h"
static inline struct splash_data * get_splash_data(int consolenr)
{
	BUG_ON(consolenr >= MAX_NR_CONSOLES);

	if (vc_cons[consolenr].d)
		return vc_cons[consolenr].d->vc_splash_data;
	
	return NULL;
}
#endif

/* --- Definitions for swapwriter */
#include <linux/blkdev.h>
#include <linux/swapops.h>
#include <linux/buffer_head.h>

#define DEVICE_ID_TYPE dev_t
#define DEVICE_BLOCK_TYPE struct block_device *
#define DEVICE_BLOCK_NONE NULL
#define SYS_IO_STRUCT struct bio
#define SYS_ZONE_TYPE long
#define SIG_BDEV_START 2
#define SWP_OFFSET swp_offset
#define SWP_TYPE swp_type
#define SWP_ENTRY swp_entry
#define UNUSED_SWAP_ENTRY(i) (!swap_info[i].swap_file)
#define SWAP_FILE_INODE(i) swap_info[i].swap_file->f_dentry->d_inode
#define SWAP_IS_PARTITION(i) (S_ISBLK(SWAP_FILE_INODE(i)->i_mode))
#define SWAP_DEVICE_ID(i) (SWAP_FILE_INODE(i)->i_rdev)
#define SWAP_DEVICE_BDEV(i) (swap_info[i].bdev)
#define BDEV_TO_DEVICE_ID(bdev) ((bdev)->bd_dev)
#define SWAP_BLOCKSIZE(bdev) (block_size(bdev))
#define RESUME_BDEV(i) swap_info[i].bdev
extern void kblockd_flush(void);
#define RUN_IO do { \
	int device; \
	for (device = 0; device < MAX_SWAPFILES; device++) \
		if (SWAP_DEVICE_BDEV(device)) \
			generic_unplug_device(bdev_get_queue(SWAP_DEVICE_BDEV(device))); \
	/* kblockd_flush(); io_schedule(); */ \
	} while(0)
#define SYNC_IO do { RUN_IO; schedule(); \
	check_shift_keys(0, NULL); } while(0)
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,3)
#define OPEN_BY_DEVNUM(device, mode) open_by_devnum(device, mode)
#else
#define OPEN_BY_DEVNUM(device, mode) \
			open_by_devnum(device, mode, BDEV_RAW)
#endif

/* --- End of definitions for swapwriter */

#define NUM_CPUS num_online_cpus()
#define MIN min
#define MAX max
#define PROCESS_SIG_MASK(p) &p->sighand->siglock
#define RECALC_SIGPENDING recalc_sigpending()
#define FOR_EACH_THREAD_TASK_STRUCTS *p, *g
#define FOR_EACH_THREAD_START do_each_thread(g, p)
#define FOR_EACH_THREAD_END while_each_thread(g, p);
#define THREAD_PREEMPT_COUNT preempt_count()
#define WAKE_UP(p) signal_wake_up(p, 0)
#define FREE_AREA_TYPE struct free_area
#define ZONE_TYPE struct zone
#define STORAGE_UNSUSPEND
#define STORAGE_SUSPEND
#define EAT_MEMORY_FLAGS GFP_ATOMIC | __GFP_NOWARN
#define CPU0_MASK (cpumask_of_cpu(0))

extern asmlinkage ssize_t sys_write(unsigned int fd, const char __user * buf, size_t count);

#define INACTIVE_LIST_FOR_EACH(procedure) \
do { \
 if (zone->nr_inactive) { \
  struct list_head * entry = zone->inactive_list.prev; \
  while (entry != &zone->inactive_list) { \
   struct page * page = list_entry(entry, struct page, lru); \
   { \
    procedure(page); \
   } \
   entry = entry->prev; \
  } \
 } \
} while(0)

#define ACTIVE_LIST_FOR_EACH(procedure) \
do { \
 if (zone->nr_active) { \
  struct list_head * entry = zone->active_list.prev; \
  while (entry != &zone->active_list) { \
   struct page * page; \
   page = list_entry(entry, struct page, lru); \
   procedure(page); \
   entry = entry->prev; \
  } \
 } \
} while(0)

extern void disable_pcp_lists(void);
extern void enable_pcp_lists(void);

#ifdef CONFIG_HIGHMEM
#define KMAP_ATOMIC(a) \
	(((highmem_start_page) && (a < highmem_start_page)) ? \
	 page_address(a) : kmap_atomic(a, KM_USER1))
#define KUNMAP_ATOMIC(a) \
	do { \
		if ((highmem_start_page) && (a >= highmem_start_page)) \
			kunmap_atomic(a, KM_USER1); \
	} while(0)
#else
#define KMAP_ATOMIC(a) page_address(a)
#define KUNMAP_ATOMIC(a) do { } while(0)
#endif

#endif
