#ifndef _LINUX_SWSUSP_H
#define _LINUX_SWSUSP_H

#ifdef CONFIG_PM

#include <linux/version.h>
#include <linux/init.h>
#include <asm/atomic.h>

#include <linux/suspend-debug.h>

extern unsigned long software_suspend_state;
struct page * get_suspend_pool_page(unsigned int gfp_mask, unsigned int order);
void free_suspend_pool_pages(struct page *page, unsigned int order);
extern char idletimeout;
extern void schedule_suspend_message(int message_number);
extern atomic_t suspend_num_active;
#define SOFTWARE_SUSPEND_DISABLED		1
#define SOFTWARE_SUSPEND_RUNNING		2
#define SOFTWARE_SUSPEND_RESUME_DEVICE_OK	4
#define SOFTWARE_SUSPEND_NORESUME_SPECIFIED	8
#define SOFTWARE_SUSPEND_COMMANDLINE_ERROR 	16
#define SOFTWARE_SUSPEND_IGNORE_IMAGE		32
#define SOFTWARE_SUSPEND_SANITY_CHECK_PROMPT	64
#define SOFTWARE_SUSPEND_FREEZE_NEW_ACTIVITY	128
#define SOFTWARE_SUSPEND_FREEZE_UNREFRIGERATED	256
#define SOFTWARE_SUSPEND_BLOCK_PAGE_ALLOCATIONS	512
#define SOFTWARE_SUSPEND_USE_MEMORY_POOL	1024
#define SOFTWARE_SUSPEND_STAGE2_CONTINUE	2048
#define SOFTWARE_SUSPEND_FREEZE_SMP		4096
#define SOFTWARE_SUSPEND_PAGESET2_NOT_LOADED	8192
#define SOFTWARE_SUSPEND_CONTINUE_REQ		16384
#define SOFTWARE_SUSPEND_RESUMED_BEFORE		32768
extern int suspend_min_free;

#ifdef CONFIG_MTRR
#define SOFTWARE_SUSPEND_MTRR 1
#else
#undef SOFTWARE_SUSPEND_MTRR
#endif

/* kernel/suspend.c */
extern void software_suspend_pending(void);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
extern void software_resume2(void);
#endif

extern int register_suspend_notifier(struct notifier_block *);
extern int unregister_suspend_notifier(struct notifier_block *);
extern int register_resume_notifier(struct notifier_block *);
extern int unregister_resume_notifier(struct notifier_block * nb);
extern void refrigerator(unsigned long);
extern struct buffer_head * suspend_swap_bh;
extern inline void cleanup_finished_suspend_io(void);
extern void __suspend_activity_start(int, const char *, const char *);
extern atomic_t suspend_num_doing_io;
#define SWSUSP_CANHOLD 1
#define SWSUSP_WAIT_ON_ME 2

#define DECLARE_SWSUSP_LOCAL_VAR \
 unsigned long suspend_orig_value = (current->flags & PF_FRIDGE_WAIT); \
 int suspend_local_paused = 0;

#define SWSUSP_THREAD_FLAGS_RESET \
 current->flags &= ~PF_FRIDGE_WAIT; \
 suspend_orig_value = 0;

#define SWSUSP_ACTIVITY_START(flags) do { \
 (void)(suspend_orig_value); \
 (void)(suspend_local_paused); \
 if (!suspend_orig_value) \
   __suspend_activity_start(flags, __FUNCTION__, __FILE__); \
} while(0)

#define SWSUSP_ACTIVITY_RESTARTING(freezer_flags) do { \
 (void)(suspend_orig_value); \
 (void)(suspend_local_paused); \
 if (suspend_local_paused) { \
  __suspend_activity_start(freezer_flags, __FUNCTION__, __FILE__); \
  suspend_local_paused = 0; \
 } \
 if (unlikely(current->flags & PF_FREEZE)) \
	refrigerator(freezer_flags); \
} while(0)

#define SWSUSP_ACTIVITY_END do { \
 if ((!suspend_orig_value) && (current->flags & PF_FRIDGE_WAIT)) { \
   current->flags &= ~PF_FRIDGE_WAIT; \
   atomic_dec(&suspend_num_active); \
 }; \
} while(0)

#define SWSUSP_ACTIVITY_PAUSING do { \
  if ((current->flags & PF_FRIDGE_WAIT) && \
      ((software_suspend_state & SOFTWARE_SUSPEND_FREEZE_UNREFRIGERATED) || \
       (!(current->flags & PF_SYNCTHREAD)))) { \
   current->flags &= ~PF_FRIDGE_WAIT; \
   atomic_dec(&suspend_num_active); \
   suspend_local_paused = 1; \
  } \
} while(0)

/* Like the above, but we decrement active count for SYNCTHREADs too.
 * Note that the thread can start new activity. Indeed, it might need
 * to when we begin our sync.
 */
#define SWSUSP_ACTIVITY_SYNCTHREAD_PAUSING do { \
  if (current->flags & PF_FRIDGE_WAIT) { \
   current->flags &= ~PF_FRIDGE_WAIT; \
   atomic_dec(&suspend_num_active); \
   suspend_local_paused = 1; \
  } \
} while(0)

extern unsigned int suspend_task;

#define TASK_FROZEN(p) ((p->flags & PF_FROZEN) || \
			((software_suspend_state & SOFTWARE_SUSPEND_FREEZE_UNREFRIGERATED) && \
			 (!(p->flags & PF_FREEZE)) && \
			 (p->pid) && \
			 (suspend_task != p->pid)))

#else
#define software_suspend_pending()	do { } while(0)
#define software_resume2()		do { } while(0)
#define register_suspend_notifier(a)	do { } while(0)
#define unregister_suspend_notifier(a)	do { } while(0)
#define refrigerator(a)			do { BUG(); } while(0)
#define DECLARE_SWSUSP_LOCAL_VAR
#define SWSUSP_ACTIVITY_START(flush)	do { } while(0)
#define SWSUSP_ACTIVITY_END		do { } while(0)
#define SWSUSP_ACTIVITY_PAUSING		do { } while(0)
#define SWSUSP_ACTIVITY_SYNCTHREAD_PAUSING do { } while(0)
#define SWSUSP_ACTIVITY_RESTARTING(flush) do { } while(0)
#define SWSUSP_THREAD_FLAGS_RESET	do { } while(0)
#define TASK_FROZEN(p) (0)
#define suspend_task (0)
#define printlog(f, a...)		do { } while(0)
#define printnolog(f, a...)		do { } while(0)
#endif

#endif /* _LINUX_SWSUSP_H */
